from cpl_core.database.context import DatabaseContextABC

from bot_data.abc.auto_role_repository_abc import AutoRoleRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.model.auto_role import AutoRole
from bot_graphql.abc.query_abc import QueryABC


class AutoRoleMutation(QueryABC):
    def __init__(
        self,
        servers: ServerRepositoryABC,
        auto_roles: AutoRoleRepositoryABC,
        db: DatabaseContextABC,
    ):
        QueryABC.__init__(self, "AutoRoleMutation")

        self._servers = servers
        self._auto_roles = auto_roles
        self._db = db

        self.set_field("create_auto_role", self.resolve_create_auto_role)
        self.set_field("update_auto_role", self.resolve_update_auto_role)
        self.set_field("delete_auto_role", self.resolve_delete_auto_role)

    def resolve_create_auto_role(self, *_, input: dict):
        auto_role = AutoRole(
            self._servers.get_server_by_id(input["server_id"]), input["channel_id"], input["message_id"]
        )
        self._auto_roles.add_auto_role(auto_role)
        self._db.save_changes()

        def get_new(x: AutoRole):
            return (
                x.server.server_id == input["server_id"]
                and x.discord_channel_id == input["channel_id"]
                and x.discord_message_id == input["message_id"]
            )

        return self._auto_roles.get_auto_roles_by_server_id(auto_role.server.server_id).where(get_new)

    def resolve_update_auto_role(self, *_, input: dict):
        auto_role = self._auto_roles.get_auto_role_by_id(input["id"])
        auto_role.discord_channel_id = input["channel_id"] if "channel_id" in input else auto_role.discord_channel_id
        auto_role.discord_message_id = input["message_id"] if "message_id" in input else auto_role.discord_message_id

        self._auto_roles.update_auto_role(auto_role)
        self._db.save_changes()

        auto_role = self._auto_roles.get_auto_role_by_id(input["id"])
        return auto_role

    def resolve_delete_auto_role(self, *_, id: int):
        auto_role = self._auto_roles.get_auto_role_by_id(id)
        self._auto_roles.delete_auto_role(auto_role)
        self._db.save_changes()
        return auto_role
