from cpl_core.database.context import DatabaseContextABC

from bot_data.abc.auto_role_repository_abc import AutoRoleRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.model.auto_role_rule import AutoRoleRule
from bot_graphql.abc.query_abc import QueryABC


class AutoRoleRuleMutation(QueryABC):
    def __init__(
        self,
        servers: ServerRepositoryABC,
        auto_roles: AutoRoleRepositoryABC,
        db: DatabaseContextABC,
    ):
        QueryABC.__init__(self, "AutoRoleRuleMutation")

        self._servers = servers
        self._auto_roles = auto_roles
        self._db = db

        self.set_field("create_auto_role_rule", self.resolve_create_auto_role_rule)
        self.set_field("update_auto_role_rule", self.resolve_update_auto_role_rule)
        self.set_field("delete_auto_role_rule", self.resolve_delete_auto_role_rule)

    def resolve_create_auto_role_rule(self, *_, input: dict):
        auto_role_rule = AutoRoleRule(
            self._auto_roles.get_auto_role_by_id(input["auto_role_id"]), input["emoji_name"], input["role_id"]
        )
        self._auto_roles.add_auto_role_rule(auto_role_rule)
        self._db.save_changes()

        def get_new(x: AutoRoleRule):
            return (
                x.auto_role.auto_role_id == input["auto_role_id"]
                and x.emoji_name == input["emoji_name"]
                and x.role_id == input["role_id"]
            )

        return self._auto_roles.get_auto_role_rules_by_auto_role_id(auto_role_rule.auto_role.auto_role_id).where(
            get_new
        )

    def resolve_update_auto_role_rule(self, *_, input: dict):
        auto_role_rule = self._auto_roles.get_auto_role_rule_by_id(input["id"])
        auto_role_rule.emoji_name = input["emoji_name"] if "emoji_name" in input else auto_role_rule.emoji_name
        auto_role_rule.role_id = input["role_id"] if "role_id" in input else auto_role_rule.role_id

        self._auto_roles.update_auto_role_rule(auto_role_rule)
        self._db.save_changes()

        auto_role_rule = self._auto_roles.get_auto_role_rule_by_id(input["id"])
        return auto_role_rule

    def resolve_delete_auto_role_rule(self, *_, id: int):
        auto_role_rule = self._auto_roles.get_auto_role_rule_by_id(id)
        self._auto_roles.delete_auto_role_rule(auto_role_rule)
        self._db.save_changes()
        return auto_role_rule
